#!python3.7
# raw_rx.py
# Copyright (C) 2013-2020 FibreCode GmbH
#
# This module is part of FibreCode software packages and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php 
'''
Date: 2020-10-12
Description: Python sample for all Faw-Rx-Frames including new Ext.Trigger Event
OS: requires Python 3.x
Requires: USB-Raw Adapter (FC611, FC612, ...)
Latest Comment: created
'''

import sys, os
import time, datetime


from broadway2 import Broadway, RawReceiver, RawRxHeaderTypes, RawRxFrame, EthFrame
from broadway2 import broadway_api

# enable TriggerLoopback to have FC1015 connected with Ch1-Ch2 loopback-cable
TriggerLoopBack=False

def main(args):
    
    # create adapter singleton
    base = Broadway()

    aList = base.get_raw_adapters()

    if len(aList) < 1:
        print('no Raw-Adapter found')
        sys.exit(-1)


    if TriggerLoopBack:
        print('Is FC1015 Trigger-Extension connected and cable between Ch1 to Ch2?')

    if len(aList) > 1:
        cnt = 1
        for a in aList:
            print('[ {0:2} ]: {1} / {2}'.format(cnt, a[1], a[0]))
            cnt += 1
        try:
            sel = int(input('Enter Number to use:'))
        except:
            sel = 0
        
    else:
        sel = 1

    if sel < 1:
        print('Wrong selection. Exit')
        sys.exit(-1)


    inst, friendlyName = aList[sel-1]    # get adapter selected by instance   
    

    rawAdapter = base.get_adapter(inst)

    dev = base.open_control(rawAdapter.Inst)
    rawAdapter.connect(dev)

    devRx = base.open_raw_rx(rawAdapter.Inst, RawRxHeaderTypes.HeaderType2)
    print('Raw Rx Device opened: {0}'.format(devRx))

    # enable idle-frames (500 ms) to receive timestamps in case of no data
    res = broadway_api.enable_idle_frames(devRx, 500)

    # enable Ext.Trigger Pin for falling edge
    res = broadway_api.config_ext_trigger(devRx, 0, 1)
    print('Enable ext. Triggenerator: {0}'.format(res))

    # enable Trig-Out Pin if EthType == ??? is rceived
    rawAdapter.config_ext_trigger_out1(1, [0x88F7], 50)

    # and finally enable the reception of raw frames
    res = broadway_api.enable_raw_rx(devRx)
    lastTrig = 0
    try:
        while True:
            
            res = broadway_api.wait_raw_rx_frame(devRx, 100)

            if res:

                while True:
                                          
                    res = broadway_api.get_raw_rx_frame_ext(devRx)

                
                    if res != None:
                        rawRx = RawRxFrame(res[0], res[1])

                        if rawRx.FrameType == broadway_api.RawRxFrameType.FCBRawFrameTypeExternalTrigger:
                            # Ext.Trigger
                            print('Ext.Trigger: Time: {0:4}.{1} Nr : {2:4} Source: {3:2} Type:{4:2}'.format(rawRx.TimestampSec, rawRx.TimestampNano, rawRx.EventNumber, rawRx.EventSource, rawRx.EventType))
                            if lastTrig > 0:
                                print('   Diff: {0} ms'.format((rawRx.get_timestamp_ns() - lastTrig) / 1e6))
                            lastTrig = rawRx.get_timestamp_ns()
                        elif rawRx.FrameType == broadway_api.RawRxFrameType.FCBRawFrameTypeData:
                            print('EthFrame:    Time: {0:4}.{1:10} Seq: {2:4} Eth: {3}'.format(rawRx.TimestampSec, rawRx.TimestampNano, rawRx.SequenceCnt, rawRx.EthFrame))
                        elif rawRx.FrameType == broadway_api.RawRxFrameType.FCBRawFrameTypeIdle:
                            print('IDLE:        Time: {0:4}.{1} Seq: {2:4}'.format(rawRx.TimestampSec, rawRx.TimestampNano, rawRx.SequenceCnt))
                            if TriggerLoopBack:
                                rawAdapter.force_trigger_out1()

                    else:
                        break

            else:
                time.sleep(0.1)
                

            
    except KeyboardInterrupt:
        print()
        res = broadway_api.disable_raw_rx(devRx)
        base.close(devRx)
        base.close(dev)



if __name__ == "__main__":
    main(sys.argv)

