#
# This file is part of python sample package for the BROADWAY2 package.
#
# MIT License

# Copyright (c) 2013-2019 FibreCode GmbH <info@fibrecode.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import sys, os
import binascii
import platform
import time, datetime

from broadway2 import Broadway, RawRxHeaderTypes, FCAdapter


def factor_to_int32(factor):
    """fixed-point decimal with 32-bit: 1.xxxxxxxx (max. 1.9999999) 
    hexVal = 0x80000000 : 1.0000
    if factor is out of valid range, default 0x80000000 returned
    """
    if factor < 0 or factor > 2:
        return 0x80000000

    return int(factor*2**31)


def int32_to_factor(intValue):
    """
    intValue 0x80000000 returns 1.0
    """
    return intValue/2**31


class TimeFactor:
    def __init__(self, factor):
        self.hexValue = 0x00

# sample for time-correction
# in this demo, PC-Time is used via Python-API -> no high precision possible

def get_correction(diff):
    res = (0,0) # (adjustFac, prec) prec: 0: ms, 1 us, 2: 100 us
    if diff < 1:
        if diff < 0.9:
            adjustFac = 0.9
            prec = 0
        elif diff < 0.95 :
            adjustFac = 0.95
            prec = 1
        elif diff < 0.99:
            adjustFac = 1-(1-diff)/4
            prec = 2
        elif diff < 0.999:
            adjustFac = 1-(1-diff)/10
            prec = 3
        else:
            adjustFac = 1-(1-diff)/20
            prec = 3
    else:
        if diff > 1.1:
            adjustFac = 1.1
            prec = 0
        elif diff > 1.05:
            adjustFac = 1.05
            prec = 1
        elif diff > 1.01:
            adjustFac = 1+(diff-1)/4
            prec = 2
        elif diff > 1.001:
            adjustFac = 1+(diff-1)/10
            prec = 3
        else:
            adjustFac = 1+(diff-1)/20
            prec = 4

    return (adjustFac, prec)

def main(args):

    # use default inst=0
    inst = 0
    

    # create adapter singleton
    base = Broadway()

    aList = base.get_raw_adapters()

    if len(aList) < 1:
        print('no USB Raw-Adapter found!')
        sys.exit(-1)

    fca = base.get_adapter(aList[0][0])

    print('Found Adapter: {0} / {1}'.format(fca.Inst, fca.FriendlyName))

    dev = base.open_control(fca.Inst)
        
    fca.connect(dev)

     # --- faster as 1

    # 1.125
    #fc61x.adjust_ptp_clock(0x90000000)

    # 1 + 0.5 + 0.25 + 0.125 = 1.875
    #fc61x.adjust_ptp_clock(0xF0000000)

    # 1 + 0.5 + 0.25 + 0.125 + .... = 1.999999999
    #fc61x.adjust_ptp_clock(0xFFFFFFFF)

    # --- slower than 1

    # 0.5 + 0.25 + 0.125 = 0.875
    #fc61x.adjust_ptp_clock(0x70000000)

    # 0.5 + 0.25 = 0.75
    #fc61x.adjust_ptp_clock(0x60000000)

    # 0.5 + 0.125 = 0.625
    #fc61x.adjust_ptp_clock(0x50000000)

    # 0.5   = 0.5
    #fc61x.adjust_ptp_clock(0x40000000)

    # 0.25 + 0.125   = 0.375
    #fc61x.adjust_ptp_clock(0x30000000)

    # correct factor from normalized session 1


    initFactor = 1.5    # Start with something uncalibrated
    fca.adjust_ptp_clock(factor_to_int32(initFactor))

    
    tm1 = time.perf_counter_ns()
    tm = fca.get_timestamp()
    hwtm1 = tm[0]*1e9+tm[1]

    try:
        while 1:
            time.sleep(1)   # wait 1 second on PC-time
    

            tm2 = time.perf_counter_ns()
            tm = fca.get_timestamp()
            hwtm2 = tm[0]*1e9+tm[1]


            print('PC-Time Diff: {0}  m:{1}'.format((tm2-tm1), (tm2-tm1)/1e9))
            print('FC61X Time-Diff: {0} m:{1}'.format((hwtm2-hwtm1), (hwtm2-hwtm1)/1e9))

            diffFactor = (tm2-tm1)/(hwtm2-hwtm1)
            print('Diff Factor: {0}'.format(diffFactor))
            # simple math lookup to correct timestamp inside FC61X
            
            adjustFac, prec = get_correction(diffFactor)
            fca.adjust_ptp_clock(factor_to_int32(adjustFac))

            err = (tm2-tm1) - (hwtm2-hwtm1)
            print('Error: {0} ns / Precision Grade: {1}'.format(err, prec))

            print('-'*60)
            print()
            

            # next cycle, get current time as ref
            tm1 = time.perf_counter_ns()
            tm = fca.get_timestamp()
            hwtm1 = tm[0]*1e9+tm[1]    
                    
    except KeyboardInterrupt:
        base.close(dev)


if __name__ == "__main__":
    main(sys.argv)
