#
# This file is part of python sample package for the BROADWAY2 package.
#
# MIT License

# Copyright (c) 2013-2019 FibreCode GmbH <info@fibrecode.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import sys
import time
import binascii
import struct

from broadway2 import Broadway, SpiAccess, SmiAccess, PhyAccess
from broadway2 import get_phy_access, get_spi_access, get_switch_access

# Please note:
# ------------------
# Use fc_control and rename FC401-Adapter connected to SJA1105SMBEVM to "SJA1105SMB"
# As broadway2-API supports multiple instances, multiple boards may be suffixed using -1, -2, ...

# Example how to control SJA1105SMBEVM via FC401 adapter. Get access to both SJA1105S via SPI and all TJA1102/100Base-TX via SMI

ToolAdapterName = 'SJA1105SMB'

def main(args):
    
    base = Broadway()

    sja1105sAdapter = base.get_adapter_by_name(ToolAdapterName)
    # verify to have just one Tool-Adapter connected
    if sja1105sAdapter != None:
        print('Use Tool-Adapter: {0} / {1}'.format(sja1105sAdapter.SerialNumber, sja1105sAdapter.FriendlyName))

        # get USB control device
        dev = base.open_control(sja1105sAdapter.Inst)

        # connect adapter instance to this control device
        sja1105sAdapter.connect(dev)

        spiAccess = get_spi_access(sja1105sAdapter) 

        sjaAccessA = get_switch_access(spiAccess, 0)
        sjaAccessB = get_switch_access(spiAccess, 1)

        switchCoreIdA = sjaAccessA.read_reg(0)
        switchCoreIdB = sjaAccessA.read_reg(0)

        print('Switch Core ID CS0: {0:08X}'.format(switchCoreIdA))
        print('Switch Core ID CS1: {0:08X}'.format(switchCoreIdB))
        
        if switchCoreIdA == 0xAE00030E:    # SJA1105S
            
            # some demo access using ACU-Registers on SW1 (CS0)

            cfgPadMii = sjaAccessA.read_regs(0x100800, 10)
            idx = 0
            for cfg in cfgPadMii:
                print('CFG_PAD_MIIx: {0} {1:08X}'.format(idx, cfg))
                idx += 1

            cfgPadMii0Tx = sjaAccessA.read_reg(0x100800)
            cfgPadMii0Rx = sjaAccessA.read_reg(0x100801)

            print('CFG_PAD_MII0_TX: {0:08X}'.format(cfgPadMii0Tx))
            print('CFG_PAD_MII0_RX: {0:08X}'.format(cfgPadMii0Rx))

            cfgPadMii0TxAdj = 0x11111111
            sjaAccessA.write_reg(0x100800, cfgPadMii0TxAdj)

            cfgPadMii = sjaAccessA.read_regs(0x100800, 10)
            for cfg in cfgPadMii:
                print('CFG_PAD_MIIx: {0:08X}'.format(cfg))

            # write back original value
            cfgPadMii0Tx = 0x12121212
            sjaAccessA.write_reg(0x100800, cfgPadMii0Tx)
        else:
            print('Unknown Device! Please verify SPI/SMI-connection')


        phy6 = get_phy_access(sja1105sAdapter, 6)
        

        print('PhyID: {0:04X}.{1:04X}'.format(phy6.read_reg(2), phy6.read_reg(1)))

        
        # close USB-Control device before exit
        base.close(dev)
    else:
        print('Tool Adapter NOT found! ', ToolAdapterName)



if __name__ == "__main__":
    main(sys.argv)



