# eth_frame.py
#
# This file is part of python wrapper for the BROADWAY2 package.
#
# MIT License

# Copyright (c) 2013-2021 FibreCode GmbH <info@fibrecode.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import binascii

class VLANType:
    """VLAN tag class for Raw-Ethernet en/decoding"""
    def __init__(self, rawData=None):
        # C-Tag
        self.C_PCP = -1   # priority code
        self.C_DEI = -1   # Drop eligible indicator
        self.C_VID = -1   # VLAN Address
        self.S_PCP = -1   # outer VLAN
        self.S_DEI = -1   # 
        self.S_VID = -1   # VLAN-Address of outer VLAN
        self.isVLAN = False
        self.isSingleVLAN = False
        self.isDoubleVLAN = False
        if rawData != None:
            try:
                vlan1 = int.from_bytes(rawData[0:2], byteorder='big')
                if vlan1 == 0x8100:  # 802.1Q Single Tag (C-Tag)
                    vlanData = int.from_bytes(rawData[2:4], byteorder='big')
                    self.C_PCP = (vlanData >> 13) & 0x7
                    self.C_DEI = (vlanData >> 12) & 0x1
                    self.C_VID = vlanData & 0xFFF
                    self.isSingleVLAN = True
                    self.isVLAN = True
                elif vlan1 == 0x88A8:  # 802.1D Double-VLAN Tag (S-Tag)
                    vlanData = int.from_bytes(rawData[2:4], byteorder='big')
                    self.S_PCP = (vlanData >> 13) & 0x7
                    self.S_DEI = (vlanData >> 12) & 0x1
                    self.S_VID = vlanData & 0xFFF

                    vlan2 = int.from_bytes(rawData[4:6], byteorder='big')
                    if vlan2 == 0x8100:
                        vlanData = int.from_bytes(rawData[6:8], byteorder='big')
                        self.C_PCP = (vlanData >> 13) & 0x7
                        self.C_DEI = (vlanData >> 12) & 0x1
                        self.C_VID = vlanData & 0xFFF
                        self.isDoubleVLAN = True
                        self.isVLAN = True


            except:
                pass

    def __str__(self):
        if self.isVLAN:
            if self.isSingleVLAN:
                return '[{0:02X}-{1:X}-{2:04X}]'.format(self.C_PCP, self.C_DEI, self.C_VID)
            elif self.isDoubleVLAN:
                return '[{0:02X}-{1:X}-{2:04X}:{3:02X}-{4:X}-{5:04X}]'.format(self.S_PCP, self.S_DEI, self.S_VID, self.C_PCP, self.C_DEI, self.C_VID)
        else:
            return ''


class EthFrame:
    """Ethernet-Frame class for Ethernet en/decoding"""
    def __init__(self, ethFrame=None):
        if ethFrame == None:
            self.TgtMac = 0
            self.SrcMac = 0
            self.EtherType = 0
            self.PCP = -1
            self.VLAN = None
            self.RTag = False
            self.RTagSeq = -1
            self.Data = []
        else:
            self.RTag = False
            if len(ethFrame) > 26:
                self.TgtMac = int.from_bytes(ethFrame[0:6], byteorder='big')
                self.SrcMac = int.from_bytes(ethFrame[6:12], byteorder='big')
                self.VLAN = VLANType(ethFrame[12:20])    # try to detect VLAN or double-VLAN
                if self.VLAN.isVLAN:
                    if self.VLAN.isSingleVLAN:
                            self.EtherType = int.from_bytes(ethFrame[16:18], byteorder='big')
                            if self.EtherType == 0xF1C1:
                                self.RTag = True
                                self.RTagSeq = int.from_bytes(ethFrame[20:22], byteorder='big')
                                self.Data = binascii.b2a_hex(ethFrame[22:])
                            else:
                                self.Data = binascii.b2a_hex(ethFrame[18:])
                    elif self.VLAN.isDoubleVLAN:
                        self.EtherType = int.from_bytes(ethFrame[20:22], byteorder='big')
                        self.Data = binascii.b2a_hex(ethFrame[22:])

                else:
                    et = int.from_bytes(ethFrame[12:14], byteorder='big')
                    if et == 0xF1C1:    # R-Tag, no VLAN
                        self.RTag = True
                        self.RTagSeq = int.from_bytes(ethFrame[16:18], byteorder='big')
                        self.EtherType = int.from_bytes(ethFrame[18:20], byteorder='big')
                        self.Data = binascii.b2a_hex(ethFrame[20:])
                    else:
                        self.EtherType = int.from_bytes(ethFrame[12:14], byteorder='big')
                        self.Data = binascii.b2a_hex(ethFrame[14:])
            else:
                pass

    def __str__(self):
        try:
            if self.VLAN.isVLAN:
                if self.RTag == True:
                    return '{0:06X}.{1:06X}.{2}.[R-{3:05}].{4:04X}'.format(self.TgtMac, self.SrcMac, self.VLAN, self.RTagSeq, self.EtherType)
                else:
                    return '{0:06X}.{1:06X}.{2}.{3:04X}'.format(self.TgtMac, self.SrcMac, self.VLAN, self.EtherType)
            else:
                if self.RTag == True:
                    return '{0:06X}.{1:06X}.[R-{2:05}].{3:04X}'.format(self.TgtMac, self.SrcMac, self.RTagSeq, self.EtherType)
                else:
                    return '{0:06X}.{1:06X}.{2:04X}'.format(self.TgtMac, self.SrcMac, self.EtherType)
        except:
            return 'N/A'