#
# This file is part of python sample package for the BROADWAY2 package.
#
# MIT License

# Copyright (c) 2013-2021 FibreCode GmbH <info@fibrecode.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


import sys, os
import platform
import time, datetime
import json
import operator


from broadway2 import Broadway, FCAdapter



AppVersion = '2.3.2'    # added support for FC621, FC631, created sections for 100T1,10T1L



class AdapterState:
    def __init__(self, a):
        self.adapter = a
        self.USBId = (a.UsbVendorId, a.UsbProductId)
        self.LinkState = False
        self.Master = False


    def print_state(self):
        print('Adapter: #{0}/{1} Name: {2} MAC: {3}'.format(self.adapter.Inst, self.adapter.DevName, self.adapter.FriendlyName, self.adapter.MACAddress))
        if self.adapter.DevName == 'FC401': # ext. Phys
            print('      PhyType: ---')
            print('      PhyRole:  ---')
            print('      LinkUp:  ---')
        else:
            if self.adapter.PhyMaster:
                role = 'Master'
            else:
                role = 'Slave'
            print('      PhyType: {0}'.format(self.adapter.PhyType))
            print('      PhyRole:  {0}'.format(role))
            print('      LinkUp:  {0}'.format(self.adapter.LinkState))
        print()

    def update(self):
        """maybe later to update more complex properties"""
        pass

class FCApp1:
    
    def __init__(self):
        self.base = Broadway()
        self.adapterState = {}

        self.adapterDict = {}   # dictionary of all found USB-Adapters
        self.connectedControl = {}     # dictionary of all connected (control)
        self.base.set_notify(self.notify_adapter_changed)

        self.update_list()

    def notify_adapter_changed(self, event):
        self.update_list()

    def update_list(self):
        self.disconnect()   # disconnect all adapters first

        self.base.scan_and_update() # rebuild list of connected adapters

        self.adapterDict = {}

        if self.base.Count == 0:
            print('No Adapters found!')
        else:
            for adapter in self.base.enumerate():
                #print('Adapter: {0:2} {1}'.format(adapter.Inst, adapter.FriendlyName))
                self.adapterDict[adapter.Inst] = adapter
            
            res = self.base.get_unsupported_adapters()

            if len(res) > 0:
                print('Unsupported devices found!')

        self.connect()  # and connect back new list

    def get_adapter_list(self):
        res = []
        for inst in self.adapterDict: 
            res.append(self.adapterDict[inst])
        return res

    def get_inst_list(self):
        res = []
        for inst in self.adapterDict: 
            res.append(inst)
        return res

    def connect(self, aList=[]):
        if aList == None:
            lst = self.get_inst_list()
        elif len(aList) == 0:
            lst = self.get_inst_list()
        else:
            lst = aList
        for inst in lst:
            dev = self.base.open_control(inst)
            if dev != None:
                a = self.adapterDict[inst]
                a.connect(dev)
                self.connectedControl[inst] = a
                aState = AdapterState(a)
                self.adapterState[inst] = aState

    def disconnect(self):
        for inst in self.connectedControl:
            try:
                a = self.adapterDict[inst]
                self.base.close(a.Device)
            except:
                pass

        self.connectedControl = {}
        self.adapterState = {}


    def update_basics(self):
        for inst in self.connectedControl:
            self.adapterState[inst].update()

    def print_basics(self):
        aList = self.get_adapter_list()
        print('Num Adapters found: {0}'.format(len(aList)))
        print('Current Connected: {0}'.format(len(self.connectedControl)))
        print('-'*48)
        for inst in self.connectedControl:
            self.adapterState[inst].print_state()
        print()

    def run(self, delay=0.1):
        while True:
            try:
                print('FC-App1: {0}/{1}'.format(AppVersion, datetime.datetime.now()))
                self.update_basics()
                self.print_basics()
                time.sleep(delay)
            except KeyboardInterrupt:
                return

        

def main(args):

    # will never return until <Ctrl-C>
    app = FCApp1()

    print('app_fc running to get all connected FC-Adapter details....')
    print('Press <Ctrl-C> to exit!')

    app.run(1)

    app.disconnect()

    print('Ctrl-C pressed. Stopped.')

if __name__ == "__main__":
    main(sys.argv)

