﻿# reg_tracker.py
#
# This file is part of python wrapper for the BROADWAY2 package.
#
# MIT License

# Copyright (c) 2013-2022 FibreCode GmbH <info@fibrecode.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import datetime
import json

class RegisterSet:
    def __init__(self):
        self.trackStart = datetime.datetime.now()
        self.trackEnd = datetime.datetime.now()
        self.regValues = {}

class RegisterSetDiff:
    def __init__(self):
        """initialize empty RegisterSet diff"""
        self.trackStart = datetime.datetime.now()
        self.trackDiffTime = datetime.datetime.now() - self.trackStart
        self.numDifference = 0
        self.regDiffValues = {}


class RegTracker():
    def __init__(self, maxTracks=-1):
        """Default constructor unlimited tracks, use arg <maxTracks> to limit"""
        self.maxTracks = maxTracks
        self.lastCompare = None
        self.RegTracks = []     #array of register sets

    def add_track(self, regSet):
        """regs is dictionary with regOffset: regValue"""
        self.RegTracks.append(regSet)
        if self.maxTracks > 1:
            if len(self.RegTracks) > self.maxTracks:
                self.RegTracks.pop(0)

    def add_track_if_changed(self, regSet):
        """regs is dictionary with regOffset: regValue"""
        if len(self.RegTracks) == 0:
            self.RegTracks.append(regSet)
        else:
            cmp = self.diff_registers(regSet, self.RegTracks[-1])   # always compare with last one in list
            if cmp.numDifference > 0:
                self.RegTracks.append(regSet)
                self.lastCompare = cmp
                return True
        return False

    def get_last_diff(self):
        """returns last compare result"""
        return self.lastCompare

    def reset(self):
        """clear all entries"""
        self.RegTracks = []

    def diff_registers(self, regSetNew, regSetCmp):
        """verify diffs in RegisterSets"""
        regDiff = RegisterSetDiff()
        regDiff.trackStart = regSetCmp.trackStart
        regDiff.trackDiffTime = regSetNew.trackEnd - regSetCmp.trackStart
        for reg in regSetCmp.regValues:
            if regSetNew.regValues[reg] != regSetCmp.regValues[reg]:
                regDiff.regDiffValues[reg] = (regSetNew.regValues[reg], regSetCmp.regValues[reg])
                regDiff.numDifference += 1
        return regDiff

    def get_num_tracks(self):
        return len(self.RegTracks)

    def get_time_range(self):
        try:
            tm0 = self.RegTracks[0].trackStart
        except:
            tm0 = 0

        try:
            tm1 = self.RegTracks[-1].trackEnd
        except:
            tm1 = 0

        return (tm0, tm1-tm0)


    def get_overall_changes(self):
        """verify changes on all entries to latest track"""
        res = []

        for track in self.RegTracks:
            if track != self.RegTracks[-1]:
                cmp = self.diff_registers(track, self.RegTracks[-1])
                if cmp.numDifference > 0:
                    res.append(cmp)

        return res


