# smi_access.py
#
# This file is part of python wrapper for the BROADWAY2 package.
#
# MIT License

# Copyright (c) 2013-2020 FibreCode GmbH <info@fibrecode.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


# generic SMI-Access via FC401 USB-Tool Adapter
# supports also FC611,FC602,FC612,FC621,FC631 using build-in Phy

# in difference to FC6xx USB-Adapters, Phy is external and unknown. Enables generic access with given PhyAddr

from broadway2 import broadway_api

class SmiAccess:
    def __init__(self, dev=None):
        """Default constructor"""
        self.__device = dev

    def get_device(self):
        """helper function to return current used device"""
        return self.__device
        
    def connect(self, dev):
        """connect to given device instance"""
        self.__device = dev

    def disconnect(self):
        """disconnect this instance from current USB HID device"""
        self.__device = None

    @property
    def isConnected(self):
        """return True/False if phy instance is connected to USB-Adapter"""
        return self.__device != None

    def read_reg(self, phyAddr, register):
        """read register access via MDIO-Clause22 on given phyAddr/register"""
        res = broadway_api.read_phy_register_ex(self.__device, phyAddr, register)
        return res

    def write_reg(self, phyAddr, register, regValue):
        """write register access via MDIO-Clause22 on given phyAddr/register"""
        res = broadway_api.write_phy_register_ex(self.__device, phyAddr, register, regValue)
        return res

    def read_reg45(self, phyAddr, mmd, register):
        """read register access via MDIO-Clause45 on given phyAddr/mmd/register"""
        res = broadway_api.read_phy_register45_ex(self.__device, phyAddr, mmd, register)
        return res

    def read_regmult45(self, phyAddr, mmd, register, numberOFRegisters):
        """read register access via MDIO-Clause45 on given phyAddr/mmd/register"""
        res = broadway_api.read_phy_multregisters45_ex(self.__device, phyAddr, mmd, register, numberOFRegisters)
        return res

    def write_reg45(self, phyAddr, mmd, register, regValue):
        """write register access via MDIO-Clause45 on given phyAddr/mmd"""
        res = broadway_api.write_phy_register45_ex(self.__device, phyAddr, mmd, register, regValue)
        return res

    def detect_phy_ids(self, phyAddr):
        # start using Clause-45
        clauseDetect = 0

        phyID1 = broadway_api.read_phy_register45_ex(self.__device, phyAddr, 0x1E, 2)
        if phyID1 <= 0 or phyID1 == 0xFFFF:
            phyID1 = broadway_api.read_phy_register_ex(self.__device, phyAddr, 2)
            if phyID1 > 0 and phyID1 < 0xFFFF:
                useC45 = False
            else:
                return (-1, -1, 0)
        else:
            useC45 = True

        if useC45:
            phyID2 = broadway_api.read_phy_register45_ex(self.__device, phyAddr, 0x1E, 3)
            if phyID2 > 0 and phyID2 < 0xFFFF:
                clauseDetect = 2
                # probe also clause 22
                phyID2x = broadway_api.read_phy_register_ex(self.__device, phyAddr, 3)
                if phyID2x != 0xFFFF and phyID2 > 0:
                    clauseDetect = 3

        else:
            phyID2 = broadway_api.read_phy_register_ex(self.__device, phyAddr, 3)
            if phyID2 > 0 and phyID2 < 0xFFFF:
                clauseDetect = 1
                

        if clauseDetect > 0:
            return (phyID1, phyID2, clauseDetect)
        else:
            return (-1, -1, 0)

    def detect(self):
        """scan all smi-adresses and check if there is valid communication
        return array of tuples (phyAddr, phyId)
        """
        res = []

        # external
        for phyAddr in range(0, 32):
            # try to read using Clause-45
            phyIds = self.detect_phy_ids(phyAddr)
            if phyIds[0] != -1:
                res.append((phyAddr, phyIds[0], phyIds[1]))

        return res

    def detect2(self):
        """scan all smi-adresses and check if there is valid communication
        return dictionary ID1, ID2, Clause: 0 (n/a), 1: Clause22, 2: Clause45, 3:Both
        """
        res = {}

        for phyAddr in range(0, 32):
            # try to read IEEE-base register
            phyIds = self.detect_phy_ids(phyAddr)
            if phyIds[0] != -1:
                res[phyAddr] = {'ID1': phyIds[0], 'ID2': phyIds[1], 'Clause': phyIds[2]}

        return res

    def disable(self):
        broadway_api.set_smi_passive(self.__device, True)

    def enable(self):
        broadway_api.set_smi_passive(self.__device, False)
                    