# spi_access.py
#
# This file is part of python wrapper for the BROADWAY2 package.
#
# MIT License

# Copyright (c) 2013-2019 FibreCode GmbH <info@fibrecode.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import struct

from broadway2 import broadway_api


class SpiAccess:
    """implement 32-bit SPI protocol for NXP Switch SJA1105x and SJA1110x
    access to SPI is supported on FC401 only, as 4 SPI lanes are only available on USB Tool-Adapter
    """
    def __init__(self, dev=None):
        """Default constructor"""
        self.__device = dev

    def get_device(self):
        """helper function to return current used device"""
        return self.__device
        
    def connect(self, dev):
        """connect to given device instance"""
        self.__device = dev

    def disconnect(self):
        """disconnect this instance from current USB HID device"""
        self.__device = None

    @property
    def isConnected(self):
        """return True/False if phy instance is connected to USB-Adapter"""
        return self.__device != None


    def set_protocol(self, protocolId):
        if protocolId == 0:
            res = broadway_api.spi_set_protocol(self.__device, broadway_api.FCBSpiProtocol.SJA1105)
            if res == 0:
                return True
        elif protocolId == 1:
            res = broadway_api.spi_set_protocol(self.__device, broadway_api.FCBSpiProtocol.SJA1110)
            if res:
                return True
        else:
            return False

    def set_speed(self, speed):
        """adjust clock-speed in range of 500kHz-12MHz. Parameter speed is in Hz"""
        res = broadway_api.spi_set_speed(self.__device, speed)
        if res == 0:
            return True

        return False

    def spi_transaction(self, cs, outData, numBytesRead, flags):
        """generic spi-transaction on given cs"""
        
        res = broadway_api.spi_transaction(self.__device, cs, outData, numBytesRead, flags)
        
        return res

    def detect(self):
        """try to detect NXP SJA11xx"""
        resList = []
        regAddr = 0 #deviceId
        for protocolId in range(0,2):
            for cs in range(0,2):
                """single 32-bit read access"""
                
                hdr = (1 << 25) & 0x7E000000
                hdr |= (regAddr << 4) & 0x01FFFFF0

                res = self.spi_transaction(cs, hdr.to_bytes(4, byteorder='big'), 4, 0)

                if res != 0x0 or res != 0xFFFFFFFF:
                    resList.append((protocolId, cs, res))

        return resList




    